%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Task Force on Modern Heuristic Optimization Test Beds
% Working Group on Modern Heuristic Optimization
% Intelligent Systems Subcommittee
% Power System Analysis, Computing, and Economic Committee
%
% 2018 Grid Optimization Competition
% Evaluating the Performance of Modern Heuristic Optimizers on Stochastic Optimization Problems applied to Smart Grids
%
% Test bed B:
% Dynamic OPF in Presence of Renewable Energy and Electric Vehicles 
%
% Based on 2014 and 2017 competition AND [1], [2]
%
% [1] J. Arvalo, F. Santos, S. Rivera, Uncertainty cost functions for solar photovoltaic generation, wind
% energy generation, and plug-in electric vehicles: Mathematical expected value and verification by monte
% carlo simulation, International Journal of Power and Energy Conversion, in press, 2018.
% http://www.inderscience.com/info/ingeneral/forthcoming.php?jcode=ijpec
% [2] S. Rivera, A. Romero, Dispatch Modeling Incorporating Maneuver Components, Wind Power and
% Electric Vehicles Penetration, using Heuristic Techniques, Application of Modern Heuristic Optimization
% Techniques in Power and Energy Systems, ed: Wiley, in press, 2018.
%
% Questions can be directed to Sergio Rivera srriverar@unal.edu.co 
%
% Incorporating basic Particle Swarm Optimization (PSO) algorithm as
% optimization engine to solve test bed declarations.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function CE_EPSO_opt(fhd,ii,jj,kk,args,mpcORIGINAL)

% Paste and adapt the following
% parts to your implementation.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
global proc
global ps
global mpc
% Reinitialization of local
% function evaluation counter.
proc.i_eval=0;
% Function evaluation at which
% the last update of the global
% best solution occurred.
% Refers to the internal evaluation
% using static penalty constraint
% handling method.
proc.last_improvement=1;
% Signalizing your
% to stop running.
proc.finish=0;

% Dimensionality of test case.
D=ps.D*6; % PLEASE NOTE THAT 6 time instances are considered

%CAPACITY LIMITS FOR EVs (Note that the condition of the EV can be Vehicle To Grid or Vehicle Battery Charge)
ps.x_min(25)=-8;
ps.x_min(36)=-8;
ps.x_min(40)=-8;
ps.x_min(53)=-8;
ps.x_max(25)=3.6;
ps.x_max(36)=3.6;
ps.x_max(40)=3.6;
ps.x_max(53)=3.6;

%CAPACITY LIMITS FOR WIND GENERATORS
ps.x_min(10)=15;
ps.x_min(12)=15;
ps.x_max(10)=150;
ps.x_max(12)=150;

%CAPACITY LIMITS FOR SOLAR GENERATORS
ps.x_min(1)=2;
ps.x_min(2)=2;
ps.x_max(1)=20;
ps.x_max(2)=20;

% Individuals' lower bounds.
x_min =[ps.x_min ps.x_min ps.x_min ps.x_min ps.x_min ps.x_min];
% Individuals' upper bounds.
x_max =[ps.x_max ps.x_max ps.x_max ps.x_max ps.x_max ps.x_max];

D_per = length( ps.x_max );

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CE Initializations
n_samples = proc.pop_size;
Nelite = 10;
sigma = 0.9;
beta = 0.9;
indexRo = floor( n_samples - Nelite );
pos = zeros( n_samples, D );
gammaIter = zeros( 1, 1e6 );

n_trials = ones( 1, D );
meanPar = zeros( 1, D );
stdPar = zeros( 1, D );

for i = 1 : D
    j = rem( i, D_per );
    if j==0
        j=D_per;
    end
    if j <= ps.D_cont % Continuous variables
        meanPar( 1, i ) = x_min( 1, i ) + rand() * ( x_max( 1, i ) - x_min( 1, i ) );
        stdPar( 1, i ) = ( x_max( 1, i ) - x_min( 1, i ) ) / 2;
    elseif j <= ps.D_cont + ps.n_OLTC % OLTC
        x_min( 1, i ) = x_min( 1, i ) - 0.499;
        x_max( 1, i ) = x_max( 1, i ) + 0.499;
        meanPar( 1, i ) = x_min( 1, i ) + rand() * ( x_max( 1, i ) - x_min( 1, i ) );
        stdPar( 1, i ) = ( x_max( 1, i ) - x_min( 1, i ) ) / 2;
    else % Shunt
        meanPar( 1, i ) = rand();
    end
end
iter = 0;
memMeanPar( iter + 1, : ) = meanPar;
memStdPar( iter + 1, : ) = stdPar;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% EPSO Initializations
global epso_par;
epso_par.pop_size = proc.pop_size * 0.5;
epso_par.beta = 0.5;
epso_par.mutation_rate = 0.7;
epso_par.communication_probability = 0.8;
% Weights matrix
% 1 - inertia
% 2 - memory
% 3 - cooperation
% 4 - perturbation
weights = rand( epso_par.pop_size, 4 );
epso_par.init = 1;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

global fitTOTAL
global objTOTAL
global g_sumTOTAL
global in_var_ROTOTAL
fitTOTAL=[];
objTOTAL=[];
g_sumTOTAL=[];
in_var_ROTOTAL=[];

gbestval = 1e30;
gbest = zeros( 1, D );

%%%%%%%
while 1
    %%%%%%%
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Run CE
    while proc.i_eval < 6 * n_samples * 5
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Generate population
        for i = 1 : D
            j = rem( i, D_per );
            if j==0
                j=D_per;
            end
            if j <= ps.D_cont + ps.n_OLTC % Continuous variables + OLTC
                tmp_stat = normrnd( meanPar( i ), stdPar( i ), 1, n_samples );
                tmp_stat( tmp_stat < x_min( 1, i ) ) = x_min( 1, i );
                tmp_stat( tmp_stat > x_max( 1, i ) ) = x_max( 1, i );
                tmp_eval = tmp_stat;
            else % Shunt compensators
                tmp_stat = binornd( n_trials( i ), meanPar( i ), 1, n_samples );
                tmp_eval = tmp_stat + x_min( 1, i );
            end
            pos( :, i ) = tmp_eval;
        end
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Evaluate population
        [ fit,~,~,~ ] = UC( fhd,ii,jj,kk,args,pos,mpcORIGINAL );
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Performs CE correction
        
        % Sorts states according to fitness
        [ Saux, Iaux ] = sort( fit, 2, 'descend' );
        H = zeros( 1, n_samples );
        for i = 1 : n_samples
            if i <= indexRo
                H( Iaux( i ) ) = 0;
            else
                H( Iaux( i ) ) = 1;
            end
        end
        
        % Computes gammaIter
        gamma = Saux( 1, indexRo );
        gammaIter( 1, iter + 1 ) = gamma;
        
        % Applies CE correction
        den = sum( H );
        for i = 1 : D
            num = H * pos( :, i );
            tmpMean = 0;
            if den > 0
                tmpMean = ( num / den ) * ( 1 / n_trials( 1, i ) );
            end
            num = H * ( ( pos( :, i ) - tmpMean ) .* ( pos( :, i ) - tmpMean ) );
            tmpStd = 0;
            if den > 0
                tmpStd = sqrt( num / den ) * ( 1 / n_trials( 1, i ) );
            end
            meanPar( i ) = tmpMean * sigma + meanPar( i ) * ( 1 - sigma );
            stdPar( i ) = tmpStd * beta + stdPar( i ) * ( 1 - beta );
            
            j = rem( i, D_per );
            if j==0
                j=D_per;
            end
            
            % Update sampling distribution parameters
            if j <= ps.n_gen_PG
                meanPar( i ) = round( meanPar( i ), 3 );
            elseif j <= ps.D_cont
                meanPar( i ) = round( meanPar( i ), 4 );
            elseif j <= ps.D_cont + ps.n_OLTC
                meanPar( i ) = round( meanPar( i ), 2 );
            else
                if meanPar( i ) < 0.01
                    meanPar( i ) = 0.01;
                elseif meanPar( i ) > 0.99
                    meanPar( i ) = 0.99;
                end
            end
        end
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % UPDATE GLOBAL BEST
        [ tmpgbestval, gbestid ] = min( fit );
        if tmpgbestval < gbestval
            gbestval = tmpgbestval;
            gbest = pos( gbestid, : );
        end
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Update CE iteration
        iter = iter + 1;
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        memMeanPar( iter + 1, : ) = meanPar;
        memStdPar( iter + 1, : ) = stdPar;
    end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Run EPSO
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %  Initialize EPSO population
    if epso_par.init == 1
        
        args{3}=epso_par.pop_size;
        
        Xmin = x_min;
        Xmax = x_max;
        Vmin = -Xmax + Xmin;
        Vmax = -Vmin;
        vel = zeros( epso_par.pop_size, D );
        pos = pos( 1:epso_par.pop_size, : );
        for i = 1 : epso_par.pop_size
            vel( i, : ) = ( Vmin + ( Vmax - Vmin ) .* rand( 1, D ) ) ;
            % pos( i, : ) = ( Xmin + ( Xmax - Xmin ) .* rand( 1, D ) ) ;
        end
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Evaluate population
        [ fit,obj,g_sum,~ ] = UC( fhd,ii,jj,kk,args,pos,mpcORIGINAL );
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Update global best
        [ gbestval, gbestid ] = min( fit );
        gbest = pos( gbestid, : );
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % UPDATE individual best
        % Individual best position ever of the particles of current population
        myBestPos = pos;
        % Fitness of the individual best position ever of the particles of current population
        myBestPosFit = fit;
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        epso_par.init = 0;
        
        gammaIter = gammaIter( 1, 1:iter );
    end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % COPY CURRENT population
    copy_pos = pos;
    copy_vel = vel;
    copy_weights = weights;
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Apply EPSO movement rule
    for i = 1 : epso_par.pop_size
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Mutate weights of the particles of the COPIED population
        copy_weights( i, : ) = MUTATE_WEIGHTS( weights( i, : ), epso_par.mutation_rate );
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Compute new velocity for the particles of the COPIED population
        copy_vel( i, : ) = EPSO_COMPUTE_NEW_VEL( D, copy_pos( i, : ), myBestPos( i, : ), gbest, ...
            copy_vel( i, : ), Vmin, Vmax, copy_weights( i, : ), epso_par.communication_probability );
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Compute new position for the particles of the COPIED population
        [ copy_pos( i, : ), copy_vel( i, : ) ] = COMPUTE_NEW_POS( copy_pos( i, : ), copy_vel( i, : ) );
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Compute new velocity for the particles of the CURRENT population
        vel( i, : ) = EPSO_COMPUTE_NEW_VEL( D, pos( i, : ), myBestPos( i, : ), gbest, ...
            vel( i, : ), Vmin, Vmax, weights( i, : ), epso_par.communication_probability );
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Compute new position for the particles of the CURRENT population
        [ pos( i, : ), vel( i, : ) ] = COMPUTE_NEW_POS( pos( i, : ), vel( i, : ) );
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
    end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Enforce search space limits of the COPIED population
    [ copy_pos, copy_vel ] = ENFORCE_POS_LIMITS( D, epso_par.pop_size, copy_pos, Xmin, Xmax, copy_vel, Vmin, Vmax );
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Enforce search space limits of the CURRENT population
    [ pos, vel ] = ENFORCE_POS_LIMITS( D, epso_par.pop_size, pos, Xmin, Xmax, vel, Vmin, Vmax );
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Evaluate the COPIED population
    [ copy_fit,copy_obj,copy_g_sum,~ ] = UC( fhd,ii,jj,kk,args,copy_pos,mpcORIGINAL );
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Evaluate population
    [ fit,obj,g_sum,~ ] = UC( fhd,ii,jj,kk,args,pos,mpcORIGINAL );
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % CREATE NEW population to replace CURRENT population
    selParNewSwarm = ( copy_fit < fit );
    for i = 1 : epso_par.pop_size
        if selParNewSwarm( i )
            fit( i ) = copy_fit( i );
            pos( i, : ) = copy_pos( i, : );
            vel( i, : ) = copy_vel( i, : );
            weights( i, : ) = copy_weights( i, : );
        end
        if fit( i ) < myBestPosFit( i )
            myBestPos( i, : ) = pos( i, : );
            myBestPosFit( i ) = fit( i );
        end
    end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % UPDATE GLOBAL BEST
    [ tmpgbestval, gbestid ] = min( fit );
    if tmpgbestval < gbestval
        gbestval = tmpgbestval;
        gbest = pos( gbestid, : );
    end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
    
    %%%%%%%%%%%%%%
    if proc.finish       
        return;
    end
    %%%%%%%%%%%%%%
    
    % To prematurely stop
    % running the current trial,
    % you may use a statement
    % as the following.
    % However, note that
    % storage of intermediate
    % results must be done
    % manually since no ASCII
    % file output will be
    % provided in that case.
    % As an example, retrieve
    % relevant information
    % from cell array res.
    %         %%%%%%%%%%%%%%%%%%%%
    %         if proc.i_eval>=1000
    %             return;
    %         end
    %         %%%%%%%%%%%%%%%%%%%%
end