%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Task Force on Modern Heuristic Optimization Test Beds
% Working Group on Modern Heuristic Optimization
% Intelligent Systems Subcommittee
% Power System Analysis, Computing, and Economic Committee
%
% 2018 Grid Optimization Competition
% Evaluating the Performance of Modern Heuristic Optimizers on Stochastic Optimization Problems applied to Smart Grids
%
% Test bed B:
% Dynamic OPF in Presence of Renewable Energy and Electric Vehicles 
%
% Based on 2014 and 2017 competition AND [1], [2]
%
% [1] J. Arvalo, F. Santos, S. Rivera, Uncertainty cost functions for solar photovoltaic generation, wind
% energy generation, and plug-in electric vehicles: Mathematical expected value and verification by monte
% carlo simulation, International Journal of Power and Energy Conversion, in press, 2018.
% http://www.inderscience.com/info/ingeneral/forthcoming.php?jcode=ijpec
% [2] S. Rivera, A. Romero, Dispatch Modeling Incorporating Maneuver Components, Wind Power and
% Electric Vehicles Penetration, using Heuristic Techniques, Application of Modern Heuristic Optimization
% Techniques in Power and Energy Systems, ed: Wiley, in press, 2018.
%
% Questions can be directed to Sergio Rivera srriverar@unal.edu.co
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% A set of Montecarlo Simulations VS EV Analytical Uncertainty Cost Function 
clc
close all

Ps=18.5;           
me=19.54;        
sg= 0.54;         
N=100000;       
Cu=300;              
Co=700;           

%% A set of montecarlo simulations
tic

ii=1;
iii=1;

CO=[];
CU=[];
Pe=[];

for i=1:N
    Pe(i)=normrnd(me,sg); 
      
    if Ps>=Pe(i)

   CO(ii)=Co*(Ps-Pe(i));
    ii=ii+1;

   else

    CU(iii)=Cu*(Pe(i)-Ps);
   iii=iii+1;

   end
       
end
   
   figure(1)
 
   subplot(3,1,1)
   hist(Pe)
   title('Histograms','color','Black','fontsize',13) , grid on;
   ylabel('Scenarios','color','Black','fontsize',9); 
   xlabel('Power [MW]','color','Black','fontsize',9);
   subplot(3,1,2)
   hist(CU)
   grid on;
   %title('Cost due to underestimate Histogram','color','Black','fontsize',12) , grid on;
   ylabel('Scenarios','color','Black','fontsize',9); 
   xlabel('Cost due to underestimate [$]','color','Black','fontsize',9);
   subplot(3,1,3)
   hist(CO)
   grid on;
   %title('Cost due to overestimate Histogram','color','Black','fontsize',12) , grid on;
   ylabel('Scenarios','color','Black','fontsize',9); 
   xlabel('Cost due overestimate [$]','color','Black','fontsize',9);
      
   %COSTO TOTAL
   
   figure(2)
   CUCO=[CU CO];
   hist(CUCO)
   title( 'UCF Histogram','color','Black','fontsize',12) , grid on;
   ylabel('Scenarios','color','Black','fontsize',10); 
   xlabel('Penalty cost (UCF) [$]','color','Black','fontsize',11);
   ExpectedUncertaintyCost_Montecarlo=mean(CUCO)
   
xo1=mean(CO);
xu1=mean(CU);
toc

%% Analytical Caculation
d=(me-Ps)/(sg*sqrt(2));

Cua=((Cu/2)*(me-Ps)*(1+erf(d)))+(((Cu*sg)/sqrt(2*pi))*exp(-d^2)); 

Coa=-((Co*sg/(sqrt(2*pi)))*((exp(-(me^2/(2*sg^2))))-(exp(-d^2))))+((Co/2)*(Ps-me)*(erf((me)/(sg*sqrt(2)))-erf(d)));

ExpectedUncertaintyCost_Analytical=Cua+Coa